@@ -22,8 +22,11 @@ use crate::{
     obj::btf::{Btf, BtfError, BtfExt},
     BpfError,
 };
+use std::slice::from_raw_parts_mut;
 
 const KERNEL_VERSION_ANY: u32 = 0xFFFF_FFFE;
+/// The first five __u32 of `bpf_map_def` must be defined.
+const MINIMUM_MAP_SIZE: usize = mem::size_of::<u32>() * 5;
 
 #[derive(Clone)]
 pub struct Object {
@@ -51,7 +54,7 @@ pub struct Map {
 pub(crate) struct Program {
     pub(crate) license: CString,
     pub(crate) kernel_version: KernelVersion,
-    pub(crate) kind: ProgramKind,
+    pub(crate) section: ProgramSection,
     pub(crate) function: Function,
 }
 
@@ -64,49 +67,86 @@ pub(crate) struct Function {
     pub(crate) instructions: Vec<bpf_insn>,
 }
 
-#[derive(Debug, Copy, Clone)]
-pub enum ProgramKind {
-    KProbe,
-    KRetProbe,
-    UProbe,
-    URetProbe,
-    TracePoint,
-    SocketFilter,
-    Xdp,
-    SkMsg,
-    SkSkbStreamParser,
-    SkSkbStreamVerdict,
-    SockOps,
-    SchedClassifier,
-    CgroupSkbIngress,
-    CgroupSkbEgress,
-    LircMode2,
+#[derive(Debug, Clone)]
+pub enum ProgramSection {
+    KRetProbe { name: String },
+    KProbe { name: String },
+    UProbe { name: String },
+    URetProbe { name: String },
+    TracePoint { name: String },
+    SocketFilter { name: String },
+    Xdp { name: String },
+    SkMsg { name: String },
+    SkSkbStreamParser { name: String },
+    SkSkbStreamVerdict { name: String },
+    SockOps { name: String },
+    SchedClassifier { name: String },
+    CgroupSkbIngress { name: String },
+    CgroupSkbEgress { name: String },
+    LircMode2 { name: String },
 }
 
-impl FromStr for ProgramKind {
+impl ProgramSection {
+    fn name(&self) -> &str {
+        match self {
+            ProgramSection::KRetProbe { name } => name,
+            ProgramSection::KProbe { name } => name,
+            ProgramSection::UProbe { name } => name,
+            ProgramSection::URetProbe { name } => name,
+            ProgramSection::TracePoint { name } => name,
+            ProgramSection::SocketFilter { name } => name,
+            ProgramSection::Xdp { name } => name,
+            ProgramSection::SkMsg { name } => name,
+            ProgramSection::SkSkbStreamParser { name } => name,
+            ProgramSection::SkSkbStreamVerdict { name } => name,
+            ProgramSection::SockOps { name } => name,
+            ProgramSection::SchedClassifier { name } => name,
+            ProgramSection::CgroupSkbIngress { name } => name,
+            ProgramSection::CgroupSkbEgress { name } => name,
+            ProgramSection::LircMode2 { name } => name,
+        }
+    }
+}
+
+impl FromStr for ProgramSection {
     type Err = ParseError;
 
-    fn from_str(kind: &str) -> Result<ProgramKind, ParseError> {
-        use ProgramKind::*;
+    fn from_str(section: &str) -> Result<ProgramSection, ParseError> {
+        use ProgramSection::*;
+
+        // parse the common case, eg "xdp/program_name" or
+        // "sk_skb/stream_verdict/program_name"
+        let mut parts = section.rsplitn(2, '/').collect::<Vec<_>>();
+        if parts.len() == 1 {
+            parts.push(parts[0]);
+        }
+        let kind = parts[1];
+        let name = parts[0].to_owned();
+
         Ok(match kind {
-            "kprobe" => KProbe,
-            "kretprobe" => KRetProbe,
-            "uprobe" => UProbe,
-            "uretprobe" => URetProbe,
-            "xdp" => Xdp,
-            "trace_point" => TracePoint,
-            "socket_filter" => SocketFilter,
-            "sk_msg" => SkMsg,
-            "sk_skb/stream_parser" => SkSkbStreamParser,
-            "sk_skb/stream_verdict" => SkSkbStreamVerdict,
-            "sockops" => SockOps,
-            "classifier" => SchedClassifier,
-            "cgroup_skb/ingress" => CgroupSkbIngress,
-            "cgroup_skb/egress" => CgroupSkbEgress,
-            "lirc_mode2" => LircMode2,
+            "kprobe" => KProbe { name },
+            "kretprobe" => KRetProbe { name },
+            "uprobe" => UProbe { name },
+            "uretprobe" => URetProbe { name },
+            "xdp" => Xdp { name },
+            _ if kind.starts_with("tracepoint") || kind.starts_with("tp") => {
+                // tracepoint sections are named `tracepoint/category/event_name`,
+                // and we want to parse the name as "category/event_name"
+                let name = section.splitn(2, '/').last().unwrap().to_owned();
+                TracePoint { name }
+            }
+            "socket_filter" => SocketFilter { name },
+            "sk_msg" => SkMsg { name },
+            "sk_skb/stream_parser" => SkSkbStreamParser { name },
+            "sk_skb/stream_verdict" => SkSkbStreamVerdict { name },
+            "sockops" => SockOps { name },
+            "classifier" => SchedClassifier { name },
+            "cgroup_skb/ingress" => CgroupSkbIngress { name },
+            "cgroup_skb/egress" => CgroupSkbEgress { name },
+            "lirc_mode2" => LircMode2 { name },
             _ => {
-                return Err(ParseError::InvalidProgramKind {
-                    kind: kind.to_string(),
+                return Err(ParseError::InvalidProgramSection {
+                    section: section.to_owned(),
                 })
             }
         })
@@ -115,7 +155,7 @@ impl FromStr for ProgramKind {
 
 impl Object {
     pub(crate) fn parse(data: &[u8]) -> Result<Object, BpfError> {
-        let obj = object::read::File::parse(data).map_err(|e| ParseError::ElfError(e))?;
+        let obj = object::read::File::parse(data).map_err(ParseError::ElfError)?;
         let endianness = obj.endianness();
 
         let license = if let Some(section) = obj.section_by_name("license") {
@@ -152,12 +192,12 @@ impl Object {
             bpf_obj.parse_section(Section::try_from(&s)?)?;
         }
 
-        return Ok(bpf_obj);
+        Ok(bpf_obj)
     }
 
     fn new(endianness: Endianness, license: CString, kernel_version: KernelVersion) -> Object {
         Object {
-            endianness: endianness.into(),
+            endianness,
             license,
             kernel_version,
             btf: None,
@@ -181,18 +221,15 @@ impl Object {
         Ok(())
     }
 
-    fn parse_program(
-        &self,
-        section: &Section,
-        ty: &str,
-        name: &str,
-    ) -> Result<Program, ParseError> {
+    fn parse_program(&self, section: &Section) -> Result<Program, ParseError> {
+        let prog_sec = ProgramSection::from_str(section.name)?;
+        let name = prog_sec.name().to_owned();
         Ok(Program {
             license: self.license.clone(),
             kernel_version: self.kernel_version,
-            kind: ProgramKind::from_str(ty)?,
+            section: prog_sec,
             function: Function {
-                name: name.to_owned(),
+                name,
                 address: section.address,
                 section_index: section.index,
                 section_offset: 0,
@@ -263,51 +300,35 @@ impl Object {
     }
 
     fn parse_section(&mut self, mut section: Section) -> Result<(), BpfError> {
-        let mut parts = section.name.rsplitn(2, "/").collect::<Vec<_>>();
+        let mut parts = section.name.rsplitn(2, '/').collect::<Vec<_>>();
         parts.reverse();
 
-        if parts.len() == 1 {
-            if parts[0] == "xdp"
+        if parts.len() == 1
+            && (parts[0] == "xdp"
                 || parts[0] == "sk_msg"
                 || parts[0] == "sockops"
-                || parts[0] == "classifier"
-            {
-                parts.push(parts[0]);
-            }
+                || parts[0] == "classifier")
+        {
+            parts.push(parts[0]);
         }
 
-        match parts.as_slice() {
-            &[name]
-                if name == ".bss" || name.starts_with(".data") || name.starts_with(".rodata") =>
-            {
+        match section.name {
+            name if name == ".bss" || name.starts_with(".data") || name.starts_with(".rodata") => {
                 self.maps
                     .insert(name.to_string(), parse_map(&section, name)?);
             }
-            &[name] if name.starts_with(".text") => self.parse_text_section(section)?,
-            &[".BTF"] => self.parse_btf(&section)?,
-            &[".BTF.ext"] => self.parse_btf_ext(&section)?,
-            &["maps", name] => {
+            name if name.starts_with(".text") => self.parse_text_section(section)?,
+            ".BTF" => self.parse_btf(&section)?,
+            ".BTF.ext" => self.parse_btf_ext(&section)?,
+            map if map.starts_with("maps/") => {
+                let name = map.splitn(2, '/').last().unwrap();
                 self.maps
                     .insert(name.to_string(), parse_map(&section, name)?);
             }
-            &[ty @ "kprobe", name]
-            | &[ty @ "kretprobe", name]
-            | &[ty @ "uprobe", name]
-            | &[ty @ "uretprobe", name]
-            | &[ty @ "socket_filter", name]
-            | &[ty @ "xdp", name]
-            | &[ty @ "trace_point", name]
-            | &[ty @ "sk_msg", name]
-            | &[ty @ "sk_skb/stream_parser", name]
-            | &[ty @ "sk_skb/stream_verdict", name]
-            | &[ty @ "sockops", name]
-            | &[ty @ "classifier", name]
-            | &[ty @ "cgroup_skb/ingress", name]
-            | &[ty @ "cgroup_skb/egress", name]
-            | &[ty @ "cgroup/skb", name]
-            | &[ty @ "lirc_mode2", name] => {
+            name if is_program_section(name) => {
+                let program = self.parse_program(&section)?;
                 self.programs
-                    .insert(name.to_string(), self.parse_program(&section, ty, name)?);
+                    .insert(program.section.name().to_owned(), program);
                 if !section.relocations.is_empty() {
                     self.relocations.insert(
                         section.index,
@@ -351,8 +372,8 @@ pub enum ParseError {
     #[error("unsupported relocation target")]
     UnsupportedRelocationTarget,
 
-    #[error("invalid program kind `{kind}`")]
-    InvalidProgramKind { kind: String },
+    #[error("invalid program section `{section}`")]
+    InvalidProgramSection { section: String },
 
     #[error("invalid program code")]
     InvalidProgramCode,
@@ -474,8 +495,7 @@ fn parse_map(section: &Section, name: &str) -> Result<Map, ParseError> {
             value_size: section.data.len() as u32,
             max_entries: 1,
             map_flags: 0, /* FIXME: set rodata readonly */
-            id: 0,
-            pinning: 0,
+            ..Default::default()
         };
         (def, section.data.to_vec())
     } else {
@@ -491,13 +511,23 @@ fn parse_map(section: &Section, name: &str) -> Result<Map, ParseError> {
 }
 
 fn parse_map_def(name: &str, data: &[u8]) -> Result<bpf_map_def, ParseError> {
-    if mem::size_of::<bpf_map_def>() > data.len() {
+    if data.len() < MINIMUM_MAP_SIZE {
         return Err(ParseError::InvalidMapDefinition {
             name: name.to_owned(),
         });
     }
 
-    Ok(unsafe { ptr::read_unaligned(data.as_ptr() as *const bpf_map_def) })
+    if data.len() < mem::size_of::<bpf_map_def>() {
+        let mut map_def = bpf_map_def::default();
+        unsafe {
+            let map_def_ptr =
+                from_raw_parts_mut(&mut map_def as *mut bpf_map_def as *mut u8, data.len());
+            map_def_ptr.copy_from_slice(data);
+        }
+        Ok(map_def)
+    } else {
+        Ok(unsafe { ptr::read_unaligned(data.as_ptr() as *const bpf_map_def) })
+    }
 }
 
 fn copy_instructions(data: &[u8]) -> Result<Vec<bpf_insn>, ParseError> {
@@ -516,6 +546,34 @@ fn copy_instructions(data: &[u8]) -> Result<Vec<bpf_insn>, ParseError> {
     Ok(instructions)
 }
 
+fn is_program_section(name: &str) -> bool {
+    for prefix in &[
+        "classifier",
+        "cgroup/skb",
+        "cgroup_skb/egress",
+        "cgroup_skb/ingress",
+        "kprobe",
+        "kretprobe",
+        "lirc_mode2",
+        "sk_msg",
+        "sk_skb/stream_parser",
+        "sk_skb/stream_verdict",
+        "socket_filter",
+        "sockops",
+        "tp",
+        "tracepoint",
+        "uprobe",
+        "uretprobe",
+        "xdp",
+    ] {
+        if name.starts_with(prefix) {
+            return true;
+        }
+    }
+
+    false
+}
+
 #[cfg(test)]
 mod tests {
     use matches::assert_matches;
@@ -609,34 +667,61 @@ mod tests {
     }
 
     #[test]
-    fn test_parse_map_def() {
+    fn test_parse_map_def_error() {
         assert!(matches!(
             parse_map_def("foo", &[]),
             Err(ParseError::InvalidMapDefinition { .. })
         ));
-        assert!(matches!(
-            parse_map_def(
-                "foo",
-                bytes_of(&bpf_map_def {
-                    map_type: 1,
-                    key_size: 2,
-                    value_size: 3,
-                    max_entries: 4,
-                    map_flags: 5,
-                    id: 0,
-                    pinning: 0
-                })
-            ),
-            Ok(bpf_map_def {
-                map_type: 1,
-                key_size: 2,
-                value_size: 3,
-                max_entries: 4,
-                map_flags: 5,
-                id: 0,
-                pinning: 0
-            })
-        ));
+    }
+
+    #[test]
+    fn test_parse_map_short() {
+        let def = bpf_map_def {
+            map_type: 1,
+            key_size: 2,
+            value_size: 3,
+            max_entries: 4,
+            map_flags: 5,
+            id: 0,
+            pinning: 0,
+        };
+
+        assert_eq!(
+            parse_map_def("foo", &bytes_of(&def)[..MINIMUM_MAP_SIZE]).unwrap(),
+            def
+        );
+    }
+
+    #[test]
+    fn test_parse_map_def() {
+        let def = bpf_map_def {
+            map_type: 1,
+            key_size: 2,
+            value_size: 3,
+            max_entries: 4,
+            map_flags: 5,
+            id: 6,
+            pinning: 7,
+        };
+
+        assert_eq!(parse_map_def("foo", bytes_of(&def)).unwrap(), def);
+    }
+
+    #[test]
+    fn test_parse_map_def_with_padding() {
+        let def = bpf_map_def {
+            map_type: 1,
+            key_size: 2,
+            value_size: 3,
+            max_entries: 4,
+            map_flags: 5,
+            id: 6,
+            pinning: 7,
+        };
+        let mut buf = [0u8; 128];
+        unsafe { ptr::write_unaligned(buf.as_mut_ptr() as *mut _, def) };
+
+        assert_eq!(parse_map_def("foo", &buf).unwrap(), def);
     }
 
     #[test]
@@ -644,7 +729,7 @@ mod tests {
         assert!(matches!(
             parse_map(&fake_section("maps/foo", &[]), "foo"),
             Err(ParseError::InvalidMapDefinition { .. })
-        ))
+        ));
     }
 
     #[test]
@@ -703,7 +788,7 @@ mod tests {
                     max_entries: 1,
                     map_flags: 0,
                     id: 0,
-                    pinning: 0
+                    pinning: 0,
                 },
                 data
             }) if name == ".bss" && data == map_data && value_size == map_data.len() as u32
@@ -723,11 +808,7 @@ mod tests {
         let obj = fake_obj();
 
         assert_matches!(
-            obj.parse_program(
-                &fake_section("kprobe/foo", &42u32.to_ne_bytes(),),
-                "kprobe",
-                "foo"
-            ),
+            obj.parse_program(&fake_section("kprobe/foo", &42u32.to_ne_bytes(),),),
             Err(ParseError::InvalidProgramCode)
         );
     }
@@ -737,11 +818,11 @@ mod tests {
         let obj = fake_obj();
 
         assert_matches!(
-            obj.parse_program(&fake_section("kprobe/foo", bytes_of(&fake_ins())), "kprobe", "foo"),
+            obj.parse_program(&fake_section("kprobe/foo", bytes_of(&fake_ins()))),
             Ok(Program {
                 license,
                 kernel_version: KernelVersion::Any,
-                kind: ProgramKind::KProbe,
+                section: ProgramSection::KProbe { .. },
                 function: Function {
                     name,
                     address: 0,
@@ -766,8 +847,7 @@ mod tests {
                     value_size: 3,
                     max_entries: 4,
                     map_flags: 5,
-                    id: 0,
-                    pinning: 0
+                    ..Default::default()
                 })
             ),),
             Ok(())
@@ -820,7 +900,7 @@ mod tests {
         assert_matches!(
             obj.programs.get("foo"),
             Some(Program {
-                kind: ProgramKind::KProbe,
+                section: ProgramSection::KProbe { .. },
                 ..
             })
         );
@@ -837,7 +917,7 @@ mod tests {
         assert_matches!(
             obj.programs.get("foo"),
             Some(Program {
-                kind: ProgramKind::UProbe,
+                section: ProgramSection::UProbe { .. },
                 ..
             })
         );
@@ -848,13 +928,25 @@ mod tests {
         let mut obj = fake_obj();
 
         assert_matches!(
-            obj.parse_section(fake_section("trace_point/foo", bytes_of(&fake_ins()))),
+            obj.parse_section(fake_section("tracepoint/foo", bytes_of(&fake_ins()))),
             Ok(())
         );
         assert_matches!(
             obj.programs.get("foo"),
             Some(Program {
-                kind: ProgramKind::TracePoint,
+                section: ProgramSection::TracePoint { .. },
+                ..
+            })
+        );
+
+        assert_matches!(
+            obj.parse_section(fake_section("tp/foo/bar", bytes_of(&fake_ins()))),
+            Ok(())
+        );
+        assert_matches!(
+            obj.programs.get("foo/bar"),
+            Some(Program {
+                section: ProgramSection::TracePoint { .. },
                 ..
             })
         );
@@ -871,7 +963,7 @@ mod tests {
         assert_matches!(
             obj.programs.get("foo"),
             Some(Program {
-                kind: ProgramKind::SocketFilter,
+                section: ProgramSection::SocketFilter { .. },
                 ..
             })
         );
@@ -888,7 +980,7 @@ mod tests {
         assert_matches!(
             obj.programs.get("foo"),
             Some(Program {
-                kind: ProgramKind::Xdp,
+                section: ProgramSection::Xdp { .. },
                 ..
             })
         );
